%title Tarot: An Informal Self-Experiment
:writing:non-fiction:tarot:spirituality:autobiography:

## Introduction

Tarot is a game of correspondences.  Rather than aiming to strictly 'divine' the future, we look for correspondences between the cards laid out before us and the rest of our lives - how this card relates to our past, what this card means in the present, and using that card to divine a new path through life.  Tarot can change the way we act, can help us search the depths of our subconscious and memories to help explain the events of the past, and will show us new ways to work with the future, ways that we would never have thought of before without some outside influence, doing so all while being impersonal, keeping our deepest secrets, and never telling a soul about these correspondences.

I got interested in Tarot through various sources.  In my exploratory high school years, I had a few friends and a few relationships with some brilliant people with diverse interests, and a few of them showed me different aspects of tarot at different points in my career.  Despite having two very scientific and skeptical parents, I developed something of an interest in occult systems such as Tarot and wound up frequenting a bookstore catering to those interests in my home town.  

At one point, while purchasing my first deck of my own, I was offered a free reading with my own deck of cards from an established cartomancer at the bookstore.  The reading proved to be one of the most powerful experiences in my life up to that point.  From then on my interest grew further, leading to many more purchases of decks and books, as well as further research online and in libraries in Tarot as well as other occult systems.  To this day, many of my readings still follow the same general plan as that original one so many years ago.

I have not forgotten my heritage, though, and the respect for the more commonly accepted sciences ingrained in me by my parents.  I thus decided to embark on a bit of an informal self-experiment, following the guidelines of the scientific method to test how working with the Tarot as a tool to explore deeper within myself might change the way I perceive the world around me, alter the way I act by forcing me to think through situations in a different way.  This experiment also acts as a bit of a slice-of-life diary, with each reading providing a view of what's going on in my life at the time - the issues that are concerning me, and all the hopes and fears that have built up to that point.

## An Informal Self-Experiment

### Questions

* How does tarot work?
* How does learning to work with the tarot affect a person?
* Does mindfully working with the tarot generally lead to a different method of dealing with events in the past?
* Does mindfully working with the tarot generally lead to a different consideration of the future?
* How does working with the tarot affect the way a person deals with events and their position in time?
* How does the conscious application of a defined set of archetypes change the way a person deals with a situation?
* How does the person respond to such questions as these above being applied to them?

What I aim to learn from this experiment is basically what would happen by changing a part of my life.  I don't want to see into the future, necessarily.  Rather, I would like to become more cognizant of the present, to take the past into account, and to integrate all levels of myself into my day-to-day life: conscious, subconscious, and unconscious.  While the questions listed above are the specific ones, the more general overview would be this: if I were to consciously try to change my outlook on life through the use of this tool, what would happen?  What would change about me as a person, not just the path my life was taking?

### Observation

> We come to the notion that the Tarot works precisely because it makes no sense.  The information exists.  Our unconscious selves already know it. What we need is a device to act as a bridge to conscious perception.[@pollack97]

The modern Tarot deck is made up of 78 cards - there are 56 minor arcana and 22 major arcana cards.  All of these cards represent different archetypes, or general ideas about different aspects of life, humanity, and the self.  More than simply divining the future, one my seek out correspondences between situations at hand, in the past, or possibly in the future through these archetypal images, laid out within the framework of the cards and how they relate to each other.  One can utilize a 'spread', a pattern in which to lay the cards with each of their positions holding a predetermined meaning so as to deepen the meanings of each card.  Just as frequently, however, one may let the cards, the reader, or the querent determine the positions of those cards, divining meanings for them as the reading progresses.

There are many different views on Tarot.  By far, the overwhelming majority of people in todays world put little or no stock in divination of any sort, and would just as soon leave any introspection in the hands of those with 'Ph.D' tacked on the ends of their names.  Of those whose opinions do not favor Tarot, there are further divisions: some may have humored a cartomancer and had a reading done for themselves and disliked it for one reason or another, having been scared by it; some may have difficulty thinking of those who deal in such things as actively harmful frauds; and everything in between.  On the other side, of course, there are those who see the cards as a window into the future, those who use them strictly for introspection, those who use them for various magical purposes, and, again, everything in between.

This document is hardly meant to be a dissertation on the cards themselves, nor the varied opinions on their meanings or acceptance in the world today.  For further information, please look up the information in the bibliography, as others can surely tell those stories far better than I.  My aim is only to explore the way that the cards can influence life.  When I began college, I saw that those around me weren't focused on growing or changing, and in fact, many had remained the same person that their parents had molded them into years back, simply refusing to change.  Not only did that idea of changing who I am in however subtle a fashion appeal to me, I felt that allowing this change instead of stagnation would mean that I was actively working to make myself a better person and doing my best to better the lives of those around me.

### Hypothesis

I feel optimistic about this experiment.  I have rarely ever taken an active role in the way my life changes, and I think that by guiding it in this small way, I can improve the way that I deal with what happens to me, improving the integration of new information in order to become a better person as a whole.  I also expect that, by being more easily able to comprehend the world around me and the people in it in reference to my own worldview, I will be able to affect those around me in a possitive manner, in effect, being a better person for them.

### Procedure

My goal is to do 78 readings of the Tarot with the goal of at least one reading per day.  For each reading, I will gather information about the background of the situation, the layout of the cards and the disposition of the deck, and finally, an analysis of the cards and the reading, including the effect that the reading had on me as a person.  These readings will be mostly for myself in my day-to-day life, but I will also attempt to read for others as well, recording the way the reading made me feel and any insights that will be applicable in my own life.

When the 78 readings are completed, I will attempt to analyze the information from the collected write-ups of the readings, looking for changes over time in the way I interacted with the world around me and how I felt as a whole.  These trends will be described in as much depth as possible in order to explore how the conscious utilization of the Tarot to change my life truly affected me and, hopefully, those around me as well.

## Experimental Data

### Preliminary Introduction

As these readings are very personal in nature, I feel the need to introduce myself before just jumping right into portions of my life, lest they make no sense at all.

I was born Matthew Joseph Scott in January, 1986 to Donna Karr and Ron Scott, two engineers with the minds of scientists.  I was raised a skeptic and an atheist.  Life was uncomplicated: everything that could be explained, was, and anything that couldn't was set aside as an unknown until it could be explained with no further thoughts on the matter.  Of course, when you tell a young child not to do something, that's a very effective way to invite them to do it in secret if possible; I was told not to concern myself with the unexplainable matters of religion, mysticism.  Thus it was that I began acquiring my secret stash of information on all of those subjects, beginning with a King James bible given to me by a camp counsellor, working up to a modest library of books dealing with topics ranging from the "big five" religions and reiki to the history and use of drugs.  I have already mentioned a little about my introductions to Tarot.

That leads to me today, August 13, 2008: I am a student in the music composition program at Colorado State University in Fort Collins, Colorado.  After trying (and failing) to be a biochemist, I've accepted that I make a far better musician and publisher than a scientist, but the need to explain, to experiment, and to experience the world around me remains.

To lend even more specifics to the situation, I was only recently accepted into the composition program after spending nearly four years in music education, finding myself disillusioned with the American public education system.  This means that my four years of paid tuition promised by my dad are up, and I must begin paying for my own tuition from my earnings.  I work at the campus library as technical support, and I am currently dealing with some problematic homophobia from my boss and coworkers there.  On top of that, just a few months ago, I underwent a very difficult breakup with my boyfriend at the time.  I "rebounded" onto another friend of mine, but as soon as we started getting close to each other, he had to move down to Denver, Colorado, an hour's drive away.  

It was this combination of stressors that lead me to look into creating my own change in life.  This jumble of emotions and ideas is what I am right now, and I want see what I can make of that by being more mindful of my life, guiding it where I can, and becoming a better person, using the Tarot as a tool.

* [2008-08-13](2008-08-13)
* [2008-08-14](2008-08-14)
* [2008-08-17](2008-08-17)
* [2008-08-18](2008-08-18)

## Analysis
The analysis of this experiment will be written when all data is collected, in order to more properly answer the questions posed. !{answer questions with experimental data}
